
/* Most are taken directly from src/internal.h or src/djgpp/interndj.h */

#define POLYGON_FIX_SHIFT     18


/* bitfield specifying which polygon attributes need interpolating */
#define INTERP_FLAT           1
#define INTERP_1COL           2
#define INTERP_3COL           4
#define INTERP_FIX_UV         8
#define INTERP_Z              16
#define INTERP_FLOAT_UV       32
#define OPT_FLOAT_UV_TO_FIX   64
#define COLOR_TO_RGB          128


/* information for polygon scanline fillers */
typedef struct POLYGON_SEGMENT
{
   fixed u, v, du, dv;              /* fixed point u/v coordinates */
   fixed c, dc;                     /* single color gouraud shade values */
   fixed r, g, b, dr, dg, db;       /* RGB gouraud shade values */
   float z, dz;                     /* polygon depth (1/z) */
   float fu, fv, dfu, dfv;          /* floating point u/v coordinates */
   unsigned char *texture;          /* the texture map */
   int umask, vmask, vshift;        /* texture map size information */
   int seg;                         /* destination bitmap selector */
} POLYGON_SEGMENT;


/* prototype for the scanline filler functions */
typedef void (*SCANLINE_FILLER)(unsigned long addr, int w, POLYGON_SEGMENT *info);


typedef struct POLYGON_EDGE      /* an active edge */
{
   int top;                      /* top y position */
   int bottom;                   /* bottom y position */
   fixed x, dx;                  /* fixed point x position and gradient */
   fixed w;                      /* width of line segment */
   POLYGON_SEGMENT dat;          /* texture/gouraud information */
   struct POLYGON_EDGE *prev;    /* doubly linked list */
   struct POLYGON_EDGE *next;
   /* from here on add-ons to the Allegro structure */
   struct POLYGON_INFO *poly;    /* father polygon */
} POLYGON_EDGE;


typedef struct POLYGON_INFO {               /* a polygon waiting rendering */
   struct POLYGON_INFO *next, *prev;        /* double linked list */
   int             inside;                  /* flag for "scanlining" */
   int             trans;                   /* is transparent */
   int             flags;                   /* INTERP_* flags */
   int             color;                   /* vtx[0]->c */
   float           a, b, c;                 /* plane's coefficients -a/d, -b/d, -c/d */
   int             alpha;                   /* belnder alpha */
   COLOR_MAP       *cmap;                   /* trans color map */
   SCANLINE_FILLER drawer, alt_drawer;      /* scanline drawing functions */
   POLYGON_EDGE    *left_edge, *right_edge; /* true edges used in interpolation */
   POLYGON_SEGMENT info;                    /* base information for scanline functions */
} POLYGON_INFO;


extern SCANLINE_FILLER _optim_alternative_drawer;
void _fill_3d_edge_structure(POLYGON_EDGE *edge, V3D *v1, V3D *v2, int flags, BITMAP *bmp);
void _fill_3d_edge_structure_f(POLYGON_EDGE *edge, V3D_f *v1, V3D_f *v2, int flags, BITMAP *bmp);
SCANLINE_FILLER _get_scanline_filler(int type, int *flags, POLYGON_SEGMENT *info, BITMAP *texture, BITMAP *bmp);
void _clip_polygon_segment(POLYGON_SEGMENT *info, int gap, int flags);

extern int _drawing_mode;

